"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GrpcClientImpl = exports.GrpcClient = exports.GrpcBridge = void 0;

var _abortController = require("abort-controller");

var _base64Js = require("base64-js");

var _reactNative = require("react-native");

var _GrpcError = require("./GrpcError");

var _GrpcCall = require("./GrpcCall");

const LINKING_ERROR = `The package '@matejdr/react-native-grpc-client' doesn't seem to be linked. Make sure: \n\n` + _reactNative.Platform.select({
  ios: "- You have run 'pod install'\n",
  default: ''
}) + '- You rebuilt the app after installing the package\n' + '- You are not using Expo managed workflow\n';
const GrpcBridge = _reactNative.NativeModules.GrpcBridge ? _reactNative.NativeModules.GrpcBridge : new Proxy({}, {
  get() {
    throw new Error(LINKING_ERROR);
  }

});
exports.GrpcBridge = GrpcBridge;
const Emitter = new _reactNative.NativeEventEmitter(_reactNative.NativeModules.GrpcBridge);

function createDeferred(signal) {
  let completed = false;
  const deferred = {};
  deferred.promise = new Promise((resolve, reject) => {
    deferred.resolve = value => {
      completed = true;
      resolve(value);
    };

    deferred.reject = reason => {
      completed = true;
      reject(reason);
    };
  });
  signal.addEventListener('abort', () => {
    if (!completed) {
      deferred.reject('aborted');
    }
  });
  return deferred;
}

let idCtr = 1;
const deferredMap = {};

function handleGrpcEvent(event) {
  var _deferred$headers, _deferred$data, _deferred$response, _deferred$trailers, _deferred$data2, _deferred$headers2, _deferred$trailers2, _deferred$response2, _deferred$data3;

  const deferred = deferredMap[event.id];

  if (deferred) {
    switch (event.type) {
      case 'headers':
        (_deferred$headers = deferred.headers) === null || _deferred$headers === void 0 ? void 0 : _deferred$headers.resolve(event.payload);
        break;

      case 'response':
        const data = (0, _base64Js.toByteArray)(event.payload);
        (_deferred$data = deferred.data) === null || _deferred$data === void 0 ? void 0 : _deferred$data.notifyData(data);
        (_deferred$response = deferred.response) === null || _deferred$response === void 0 ? void 0 : _deferred$response.resolve(data);
        break;

      case 'trailers':
        (_deferred$trailers = deferred.trailers) === null || _deferred$trailers === void 0 ? void 0 : _deferred$trailers.resolve(event.payload);
        (_deferred$data2 = deferred.data) === null || _deferred$data2 === void 0 ? void 0 : _deferred$data2.notifyComplete();
        delete deferredMap[event.id];
        break;

      case 'error':
        const error = new _GrpcError.GrpcError(event.error, event.code);
        (_deferred$headers2 = deferred.headers) === null || _deferred$headers2 === void 0 ? void 0 : _deferred$headers2.reject(error);
        (_deferred$trailers2 = deferred.trailers) === null || _deferred$trailers2 === void 0 ? void 0 : _deferred$trailers2.reject(error);
        (_deferred$response2 = deferred.response) === null || _deferred$response2 === void 0 ? void 0 : _deferred$response2.reject(error);
        (_deferred$data3 = deferred.data) === null || _deferred$data3 === void 0 ? void 0 : _deferred$data3.noitfyError(error);
        delete deferredMap[event.id];
        break;
    }
  }
}

function getId() {
  return idCtr++;
}

class GrpcClient {
  constructor() {
    Emitter.addListener('grpc-call', handleGrpcEvent);
  }

  destroy() {
    Emitter.removeAllListeners('grpc-call');
  }

  startCall(config, method, requestHeaders, methodType) {
    const id = getId();
    const abort = new _abortController.AbortController();
    abort.signal.addEventListener('abort', () => {
      GrpcBridge.cancelGrpcCall(id);
    });
    const headers = createDeferred(abort.signal);
    const trailers = createDeferred(abort.signal);
    const stream = new _GrpcCall.ServerOutputStream();
    deferredMap[id] = {
      headers,
      trailers,
      data: stream
    };
    GrpcBridge.startCall(id, config, method, requestHeaders || {}, methodType);
    const call = new _GrpcCall.GrpcCall(method, requestHeaders || {}, headers.promise, stream, trailers.promise, abort, data => {
      const requestData = (0, _base64Js.fromByteArray)(data);
      const obj = {
        data: requestData
      };
      return GrpcBridge.sendMessage(id, obj);
    }, () => {
      return GrpcBridge.finishClientStreaming(id);
    });
    call.then(result => result, () => abort.abort());
    return call;
  }

}

exports.GrpcClient = GrpcClient;
const GrpcClientImpl = new GrpcClient();
exports.GrpcClientImpl = GrpcClientImpl;
//# sourceMappingURL=GrpcClient.js.map