import type { AbortController } from 'abort-controller';
import type { CompletedGrpcCall, GrpcMetadata, GrpcServerOutputStream, ServerOutputEvent, ServerOutputEventCallback } from './types';
export declare class GrpcCall implements PromiseLike<CompletedGrpcCall> {
    #private;
    readonly method: string;
    readonly requestHeaders: GrpcMetadata;
    readonly headers: Promise<GrpcMetadata>;
    readonly responses: GrpcServerOutputStream;
    readonly trailers: Promise<GrpcMetadata>;
    readonly onSendMessage: (data: Uint8Array) => Promise<boolean>;
    readonly onFinishSendMessage: () => Promise<void>;
    private isFinished;
    constructor(method: string, requestHeaders: GrpcMetadata, headers: Promise<GrpcMetadata>, responses: ServerOutputStream, trailers: Promise<GrpcMetadata>, abort: AbortController, onSendMessage: (data: Uint8Array) => Promise<boolean>, onFinishSendMessage: () => Promise<void>);
    then<TResult1 = CompletedGrpcCall, TResult2 = unknown>(onfulfilled?: ((value: CompletedGrpcCall) => TResult1 | PromiseLike<TResult1>) | null, onrejected?: ((reason: any) => TResult2 | PromiseLike<TResult2>) | null): PromiseLike<TResult1 | TResult2>;
    sendMessage(data: Uint8Array): Promise<boolean>;
    finishSendMessage(): Promise<void> | void;
    cancel(): void;
    private completedPromise;
}
export declare class ServerOutputStream implements GrpcServerOutputStream {
    #private;
    on<T extends ServerOutputEvent>(event: T, callback: ServerOutputEventCallback<T>): () => void;
    notifyData(data: Uint8Array): void;
    notifyComplete(): void;
    noitfyError(reason: any): void;
}
