export declare type GrpcMetadata = Record<string, string>;
export declare type GrpcConfig = {
    host: string;
    isInsecure: boolean;
};
export declare type RemoveListener = () => void;
export declare type DataCallback = (data: Uint8Array) => void;
export declare type ErrorCallback = (reason: any) => void;
export declare type CompleteCallback = () => void;
export declare type ServerOutputEvent = 'data' | 'error' | 'complete';
export declare type ServerOutputEventCallback<T> = T extends 'data' ? DataCallback : T extends 'complete' ? CompleteCallback : T extends 'error' ? ErrorCallback : never;
export interface GrpcServerOutputStream {
    on<T extends ServerOutputEvent>(event: T, callback: ServerOutputEventCallback<T>): RemoveListener;
}
export declare type CompletedGrpcCall = {
    readonly method: string;
    readonly requestHeaders: GrpcMetadata;
    readonly headers?: GrpcMetadata;
    readonly responses?: GrpcServerOutputStream;
    readonly status?: number;
    readonly trailers?: GrpcMetadata;
    readonly onSendMessage?: (data: Uint8Array) => Promise<boolean>;
    readonly onFinishSendMessage?: () => Promise<void>;
};
export declare type GrpcMethodType = 'unary' | 'serverStreaming' | 'clientStreaming' | 'bidiStreaming';
